==> gtk_pizza_src1.0.tgz: pizza_utils.c <==
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include "data_entry.h"
#include <unistd.h>
#include <libpq-fe.h>
#include <ctype.h>
#include <gdk/gdkkeysyms.h>
#include <gdk_imlib.h>
#include <time.h>

/*----------------------------------------------------------------------------------------------------
Global variables for postgres connection
----------------------------------------------------------------------------------------------------*/

char	*pghost, *pgport, *pgoptions, *pgtty;
char    *dbName;
PGconn  *conn;
PGresult *res;
 
/*----------------------------------------------------------------------------------------------------
Global variables for known windows
----------------------------------------------------------------------------------------------------*/

DataEntry *login_screen;
int current_store;			// which store are we in ?

/*----------------------------------------------------------------------------------------------------
::get_new_customer_number

generate a new customer number
----------------------------------------------------------------------------------------------------*/

int get_new_customer_number ()
{
	int retVal;

	res = execSQL ("select next_custnum from stores where id = %d", current_store);
	retVal = atoi (PQgetvalue (res, 0, 0));
	PQclear (res);

	res = execSQL ("update stores set next_custnum = %d where id = %d",
		retVal+1, current_store);
	PQclear (res);

	return retVal;
}

int get_new_order_number ()
{
	int retVal;

	res = execSQL ("select next_ordernum from stores where id = %d", current_store);
	retVal = atoi (PQgetvalue (res, 0, 0));
	PQclear (res);

	res = execSQL ("update stores set next_ordernum = %d where id = %d",
		retVal+1, current_store);
	PQclear (res);

	return retVal;
}

/*----------------------------------------------------------------------------------------------------
::exit_nicely

quit nicely from postgres
----------------------------------------------------------------------------------------------------*/

void exit_nicely (PGconn *conn)
{
    PQfinish(conn);
    exit(1);
}
 
/*----------------------------------------------------------------------------------------------------
::new_pixmap

create a pixmap from a file
----------------------------------------------------------------------------------------------------*/

GtkWidget* new_pixmap (char *filename, GdkWindow *window, GdkColor *background)
{
  GtkWidget *wpixmap;
  GdkPixmap *pixmap;
  GdkBitmap *mask;

  pixmap = gdk_pixmap_create_from_xpm (window, &mask,
                       background,
                       filename);
  wpixmap = gtk_pixmap_new (pixmap, mask);

  return wpixmap;
}

GtkWidget* new_image (char *filename, GdkWindow *window, GdkColor *background)
{
  GtkWidget *wpixmap;
  GdkImlibImage *im;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  int w,h;

  printf ("Loading %s\n", filename);
  im = gdk_imlib_load_image (filename);
  w = im->rgb_width;
  h = im->rgb_height;
  gdk_imlib_render (im, w,h);
  pixmap = gdk_imlib_move_image (im);
  mask = gdk_imlib_move_mask (im);

  wpixmap = gtk_pixmap_new (pixmap, mask);

  return wpixmap;
}

void cache_images (char **filename)
{
	while (*filename)
	{
  		printf ("Loading %s ..", *filename); fflush (stdout);
		gdk_imlib_load_image (*filename);
		printf ("\n");
		filename++;
	}
}

void reload_image (GtkPixmap *widget, char *filename)
{
  GdkImlibImage *im;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  int w,h;

	g_return_if_fail (GTK_IS_PIXMAP(widget));

  im = gdk_imlib_load_image (filename);
  w = im->rgb_width;
  h = im->rgb_height;
  gdk_imlib_render (im, w,h);
  pixmap = gdk_imlib_move_image (im);
  mask = gdk_imlib_move_mask (im);

  gtk_pixmap_set (widget, pixmap, mask);
}

char *trim (char *str)
{
	char *ptr;

	ptr = str + strlen(str) - 1;
	while (ptr > str)
	{
		switch (*ptr)
		{
			case ' ':
			case '\t':
			case '\n':
				*ptr = 0;
				break;
			default:
				return str;
				break;
		}
		ptr--;
	}

	return str;
}

char *SQL(char *str)
{
	char *retVal = str;
	char *ptr;

	if (!str || !strlen(str)) return str;

	while (*str)
	{
		switch (*str)
		{
			case '\\':
			case '\'':
				*str = 0;
				ptr = str+1;
				while (*ptr)
				{
					*(ptr-1) = *ptr;
					ptr++;
				}
				break;
		}
		str++;
	}

	// now trim it
	return trim(retVal);
}

// turn an SQL date into a nice date
char *nice_date (char *str)
{
	char buffer[64];

	if (!str || !*str) return str;

	if (!memcmp (str, "epoch", 5))
	{
		*str = 0;
		return str;
	}

	memcpy (buffer, str, 16);
	buffer[16] = ' ';
	memcpy (buffer+17, str+20, 4);
	buffer[21] = 0;

	memcpy (str, buffer, 22);
	return str;
}

void message_box (char *str)
{
	static GtkWidget *dialog_window = NULL;
  	static GtkWidget *label;
  	GtkWidget *button;

	if (!dialog_window)
    {
		dialog_window = gtk_dialog_new ();

		gtk_signal_connect (GTK_OBJECT (dialog_window), "destroy",
			GTK_SIGNAL_FUNC(gtk_widget_destroyed),
			&dialog_window);

		gtk_window_set_title (GTK_WINDOW (dialog_window), "Message");
		gtk_container_border_width (GTK_CONTAINER (dialog_window), 0);

		button = gtk_button_new_with_label ("OK");
		GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->action_area), 
			button, TRUE, TRUE, 0);
		gtk_widget_grab_default (button);
		gtk_widget_show (button);
		gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC(gtk_widget_destroy),
			GTK_OBJECT (dialog_window));
										 
		label = gtk_label_new (str);
		gtk_misc_set_padding (GTK_MISC (label), 10, 10);
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog_window)->vbox),
			label, TRUE, TRUE, 0);
		gtk_widget_show (label);
    }

	gtk_label_set (GTK_LABEL(label), str);

	if (!GTK_WIDGET_VISIBLE (dialog_window))
		gtk_widget_show (dialog_window);
	else
		gtk_widget_destroy (dialog_window);
}

char *strupr (char *str)
{
	char *ptr = str;
	if (!str) return str;

	while (*str)
		*str = toupper (*str++);
	return ptr;
}

char *integer (float f, int len)
{
	static char text[32];
	static char vtext[32];
	int slen, offset;
	char *ptr;

	memset (text, ' ', 32);
	text[31] = 0;
	sprintf (vtext, "%*.0f", len, f);
	ptr = strchr (vtext, '.');
	if (ptr) *ptr = 0;
	slen = strlen (vtext);

	offset = len - slen;
	if (offset < 0) offset = 0;
	strcpy (text + offset, vtext);

	return text;
}

char *money (float f, int len)
{
	static char text[32];
	static char vtext[32];
	int slen, offset;

	memset (text, ' ', 32);
	text[31] = 0;
	sprintf (vtext, "%.02f", f);
	slen = strlen (vtext);

	if (vtext[slen - 2] == '.') strcat (vtext, "0");
	offset = len - slen;
	if (offset < 0) offset = 0;
	strcpy (text + offset, vtext);

	return text;
}

/*---------------------------------------------
	Wrapper function for SQL statements
----------------------------------------------*/

PGresult *execSQL (char *format, ...)
{
	va_list ap;
	static char command[1024];
	PGresult *retval;
	time_t t;
	int nTuples;

	t = time(NULL);
	
	va_start (ap, format);
	vsprintf (command, format, ap);
	printf ("%s\t- '%s'\n", ctime (&t), command);
	retval = PQexec (conn, command);

	if (!retval)
	{
		printf ("\t  *** ERROR ***\n\n");
		message_box ("A database error has occured - check log file !\n");
		return NULL;
	}
	if (!memcmp (command, "select", 6) && (PQresultStatus(retval) != PGRES_TUPLES_OK))
	{
		printf ("\t  *** ERROR (%d) ***\n", PQresultStatus (retval));
		message_box ("A database error has occured - check log file !\n");
	}
	else if (memcmp (command, "select", 6))
	{
		nTuples = PQresultStatus (retval);
		printf ("\t  (%d updates)\n\n", nTuples);
	}
	else
	{
		nTuples = PQntuples (retval);
		printf ("\t  (%d Rows)\n\n", nTuples);
	}

	va_end (ap);
	return retval;
}

==> gtk_pizza_src1.2.tgz: pizza_utils.c <==
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include "data_entry.h"
#include <unistd.h>
#include <libpq-fe.h>
#include <ctype.h>
#include <gdk/gdkkeysyms.h>
#include <gdk_imlib.h>
#include <time.h>

extern char pixdir[];

/*----------------------------------------------------------------------------------------------------
Global variables for postgres connection
----------------------------------------------------------------------------------------------------*/

char	*pghost, *pgport, *pgoptions, *pgtty;
char    *dbName;
PGconn  *conn;
PGresult *res;
 
/*----------------------------------------------------------------------------------------------------
Global variables for known windows
----------------------------------------------------------------------------------------------------*/

DataEntry *login_screen;
int current_store;			// which store are we in ?

/*----------------------------------------------------------------------------------------------------
::get_new_customer_number

generate a new customer number
----------------------------------------------------------------------------------------------------*/

int get_new_customer_number ()
{
	int retVal;

	res = execSQL ("select next_custnum from stores where id = %d", current_store);
	retVal = atoi (PQgetvalue (res, 0, 0));
	PQclear (res);

	res = execSQL ("update stores set next_custnum = %d where id = %d",
		retVal+1, current_store);
	PQclear (res);

	return retVal;
}

int get_new_order_number ()
{
	int retVal;

	res = execSQL ("select next_ordernum from stores where id = %d", current_store);
	retVal = atoi (PQgetvalue (res, 0, 0));
	PQclear (res);

	res = execSQL ("update stores set next_ordernum = %d where id = %d",
		retVal+1, current_store);
	PQclear (res);

	return retVal;
}

/*----------------------------------------------------------------------------------------------------
::exit_nicely

quit nicely from postgres
----------------------------------------------------------------------------------------------------*/

void exit_nicely (PGconn *conn)
{
    PQfinish(conn);
    exit(1);
}
 
/*----------------------------------------------------------------------------------------------------
::new_pixmap

create a pixmap from a file
----------------------------------------------------------------------------------------------------*/

GtkWidget* new_pixmap (char *filename, GdkWindow *window, GdkColor *background)
{
  GtkWidget *wpixmap;
  GdkPixmap *pixmap;
  GdkBitmap *mask;

  pixmap = gdk_pixmap_create_from_xpm (window, &mask,
                       background,
                       filename);
  wpixmap = gtk_pixmap_new (pixmap, mask);

  return wpixmap;
}

GtkWidget* new_image (char *filename, GdkWindow *window, GdkColor *background)
{
  GtkWidget *wpixmap;
  GdkImlibImage *im;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  int w,h;
  static char try_filename[64];
  FILE *tryfp;

  sprintf (try_filename, "%s/%s", pixdir, filename);
  tryfp = fopen (try_filename, "r");
  if (!tryfp)
  {
  	strcpy (try_filename, filename);
	tryfp = fopen (try_filename, "r");
	if (!tryfp)
	{
		printf ("Cannot open %s in either . or ./%s\n", filename, pixdir);
		return NULL;
	}
	else
	{
		fclose (tryfp);
	}
  }
  else
  {
  	fclose (tryfp);
  }

  printf ("Loading %s\n", try_filename);

  im = gdk_imlib_load_image (try_filename);
  w = im->rgb_width;
  h = im->rgb_height;
  gdk_imlib_render (im, w,h);
  pixmap = gdk_imlib_move_image (im);
  mask = gdk_imlib_move_mask (im);

  wpixmap = gtk_pixmap_new (pixmap, mask);

  return wpixmap;
}

void cache_images (char **filename)
{
	while (*filename)
	{
  		printf ("Loading %s ..", *filename); fflush (stdout);
		gdk_imlib_load_image (*filename);
		printf ("\n");
		filename++;
	}
}

void reload_image (GtkPixmap *widget, char *filename)
{
  GdkImlibImage *im;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  int w,h;
  static char try_filename[64];
  FILE *tryfp;

  sprintf (try_filename, "%s/%s", pixdir, filename);
  tryfp = fopen (try_filename, "r");
  if (!tryfp)
  {
  	strcpy (try_filename, filename);
	tryfp = fopen (try_filename, "r");
	if (!tryfp)
	{
		printf ("Cannot open %s in either . or ./%s\n", filename, pixdir);
		return NULL;
	}
	else
	{
		fclose (tryfp);
	}
  }
  else
  {
  	fclose (tryfp);
  }

	g_return_if_fail (GTK_IS_PIXMAP(widget));

  im = gdk_imlib_load_image (try_filename);
  w = im->rgb_width;
  h = im->rgb_height;
  gdk_imlib_render (im, w,h);
  pixmap = gdk_imlib_move_image (im);
  mask = gdk_imlib_move_mask (im);

  gtk_pixmap_set (widget, pixmap, mask);
}

char *trim (char *str)
{
	char *ptr;

	ptr = str + strlen(str) - 1;
	while (ptr > str)
	{
		switch (*ptr)
		{
			case ' ':
			case '\t':
			case '\n':
				*ptr = 0;
				break;
			default:
				return str;
				break;
		}
		ptr--;
	}

	return str;
}

char *SQL(char *str)
{
	char *retVal = str;
	char *ptr;

	if (!str || !strlen(str)) return str;

	while (*str)
	{
		switch (*str)
		{
			case '\\':
			case '\'':
				*str = 0;
				ptr = str+1;
				while (*ptr)
				{
					*(ptr-1) = *ptr;
					ptr++;
				}
				break;
		}
		str++;
	}

	// now trim it
	return trim(retVal);
}

// turn an SQL date into a nice date
char *nice_date (char *str)
{
	char buffer[64];

	if (!str || !*str) return str;

	if (!memcmp (str, "epoch", 5))
	{
		*str = 0;
		return str;
	}

	memcpy (buffer, str, 16);
	buffer[16] = ' ';
	memcpy (buffer+17, str+20, 4);
	buffer[21] = 0;

	memcpy (str, buffer, 22);
	return str;
}

void message_box (char *str)
{
	static GtkWidget *dialog_window = NULL;
  	static GtkWidget *label;
  	GtkWidget *button;

	if (!dialog_window)
    {
		dialog_window = gtk_dialog_new ();

		gtk_signal_connect (GTK_OBJECT (dialog_window), "destroy",
			GTK_SIGNAL_FUNC(gtk_widget_destroyed),
			&dialog_window);

		gtk_window_set_title (GTK_WINDOW (dialog_window), "Message");
		gtk_container_border_width (GTK_CONTAINER (dialog_window), 0);

		button = gtk_button_new_with_label ("OK");
		GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_window)->action_area), 
			button, TRUE, TRUE, 0);
		gtk_widget_grab_default (button);
		gtk_widget_show (button);
		gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC(gtk_widget_destroy),
			GTK_OBJECT (dialog_window));
										 
		label = gtk_label_new (str);
		gtk_misc_set_padding (GTK_MISC (label), 10, 10);
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dialog_window)->vbox),
			label, TRUE, TRUE, 0);
		gtk_widget_show (label);
    }

	gtk_label_set (GTK_LABEL(label), str);

	if (!GTK_WIDGET_VISIBLE (dialog_window))
		gtk_widget_show (dialog_window);
	else
		gtk_widget_destroy (dialog_window);
}

char *strupr (char *str)
{
	char *ptr = str;
	if (!str) return str;

	while (*str)
		*str = toupper (*str++);
	return ptr;
}

char *integer (float f, int len)
{
	static char text[32];
	static char vtext[32];
	int slen, offset;
	char *ptr;

	memset (text, ' ', 32);
	text[31] = 0;
	sprintf (vtext, "%*.0f", len, f);
	ptr = strchr (vtext, '.');
	if (ptr) *ptr = 0;
	slen = strlen (vtext);

	offset = len - slen;
	if (offset < 0) offset = 0;
	strcpy (text + offset, vtext);

	return text;
}

char *money (float f, int len)
{
	static char text[32];
	static char vtext[32];
	int slen, offset;

	memset (text, ' ', 32);
	text[31] = 0;
	sprintf (vtext, "%.02f", f);
	slen = strlen (vtext);

	if (vtext[slen - 2] == '.') strcat (vtext, "0");
	offset = len - slen;
	if (offset < 0) offset = 0;
	strcpy (text + offset, vtext);

	return text;
}

/*---------------------------------------------
	Wrapper function for SQL statements
----------------------------------------------*/

PGresult *execSQL (char *format, ...)
{
	va_list ap;
	static char command[1024];
	PGresult *retval;
	time_t t;
	int nTuples;

	t = time(NULL);
	
	va_start (ap, format);
	vsprintf (command, format, ap);
	printf ("%s\t- '%s'\n", ctime (&t), command);
	retval = PQexec (conn, command);

	if (!retval)
	{
		printf ("\t  *** ERROR ***\n\n");
		message_box ("A database error has occured - check log file !\n");
		return NULL;
	}
	if (!memcmp (command, "select", 6) && (PQresultStatus(retval) != PGRES_TUPLES_OK))
	{
		printf ("\t  *** ERROR (%d) ***\n", PQresultStatus (retval));
		message_box ("A database error has occured - check log file !\n");
	}
	else if (memcmp (command, "select", 6))
	{
		nTuples = PQresultStatus (retval);
		printf ("\t  (%d updates)\n\n", nTuples);
	}
	else
	{
		nTuples = PQntuples (retval);
		printf ("\t  (%d Rows)\n\n", nTuples);
	}

	va_end (ap);
	return retval;
}
